﻿local DebuffTypeColor = DebuffTypeColor
local UnitGUID, UnitDebuff = UnitGUID, UnitDebuff

local STATUS_NAME = "alert_dispellableDebuffs"
local canDispelMagic

local GridRoster = Grid:GetModule("GridRoster")

local module = Grid:NewStatusModule("GridStatusDispellableDebuffs")
module.options = false

module.defaultDB = {
	[STATUS_NAME] = {
		text = "Dispellable debuffs",
		enable = true,
		priority = 90,
		range = false,
	},
}

local options = {
	color = false,
}

function module:PostInitialize()
	self:RegisterStatus(STATUS_NAME, "Dispellable debuffs", options, true)
end

function module:OnStatusEnable(status)
	if status == STATUS_NAME then
		self:RegisterMessage("Grid_UnitJoined")
		self:RegisterEvent("UNIT_AURA", "ScanUnitAuras")
		self:RegisterEvent("PLAYER_TALENT_UPDATE", "CheckTalents")

		self:CheckTalents()
	end
end

function module:OnStatusDisable(status)
	if status == STATUS_NAME then
		self:UnregisterMessage("Grid_UnitJoined")
		self:UnregisterEvent("UNIT_AURA")
		self:UnregisterEvent("PLAYER_TALENT_UPDATE")

		self.core:SendStatusLostAllUnits(status)
	end
end

function module:Grid_UnitJoined(event, unitGUID, unitID)
	self:ScanUnitAuras(event, unitID, unitGUID)
end

function module:UpdateAllUnitAuras()
	for unitGUID, unitID in GridRoster:IterateRoster() do
		self:ScanUnitAuras("UpdateAllUnitAuras", unitID, unitGUID)
	end
end

function module:CheckTalents()
	local currentSpec = GetSpecialization()
	canDispelMagic = currentSpec and GetSpecializationRole(currentSpec) == "HEALER"
	self:UpdateAllUnitAuras()
end

local debuffTypePriority = {
	Magic = 1,
	Curse = 2,
	Poison = 3,
	Disease = 4,
}

function module:ScanUnitAuras(event, unitID, unitGUID)
	local db = self.db.profile[STATUS_NAME]

	local guid = unitGUID or UnitGUID(unitID)

	local debuffTypeDisp, iconDisp, countDisp, durationDisp, startDisp
	local index = 1
	while true do
		local name, _, icon, count, debuffType, duration, expirationTime = UnitDebuff(unitID, index, "RAID")

		if not name then
			self.core:SendStatusLost(guid, STATUS_NAME)
			break
		end
		
		if (debuffType ~= "Magic" or canDispelMagic) and (not debuffTypeDisp or (debuffTypePriority[debuffType] < debuffTypePriority[debuffTypeDisp])) then
			-- iconDisp = "Interface\\RaidFrame\\Raid-Icon-Debuff"..debuffType
			iconDisp, countDisp, debuffTypeDisp, durationDisp, startDisp = icon, count, debuffType, duration, expirationTime - duration
		end
		index = index + 1
	end
	
	if debuffTypeDisp then
		self.core:SendStatusGained(guid, STATUS_NAME, db.priority, (db.range and 40), DebuffTypeColor[debuffTypeDisp], nil, nil, nil, iconDisp, startDisp, durationDisp, countDisp)
	end
end